@@ -7,6 +7,7 @@
 -- Portability :  portable
 
 {-# LANGUAGE FlexibleContexts #-}
+{-# LANGUAGE DeriveAnyClass #-}
 {-# LANGUAGE DeriveGeneric #-}
 module Cryptol.ModuleSystem.Monad where
 
@@ -34,7 +35,7 @@ import Data.Maybe (isJust)
 import MonadLib
 
 import GHC.Generics (Generic)
-import Control.DeepSeq.Generics
+import Control.DeepSeq
 
 import Prelude ()
 import Prelude.Compat
@@ -44,9 +45,7 @@ import Prelude.Compat
 data ImportSource
   = FromModule P.ModName
   | FromImport (Located P.Import)
-    deriving (Show,Generic)
-
-instance NFData ImportSource where rnf = genericRnf
+    deriving (Show, Generic, NFData)
 
 instance Eq ImportSource where
   (==) = (==) `on` importedModule
@@ -208,9 +207,7 @@ duplicateModuleName name path1 path2 =
 data ModuleWarning
   = TypeCheckWarnings [(Range,T.Warning)]
   | RenamerWarnings [RenamerWarning]
-    deriving (Show,Generic)
-
-instance NFData ModuleWarning where rnf = genericRnf
+    deriving (Show, Generic, NFData)
 
 instance PP ModuleWarning where
   ppPrec _ w = case w of
@@ -280,7 +277,7 @@ runModuleT :: Monad m
            => ModuleEnv
            -> ModuleT m a
            -> m (Either ModuleError (a, ModuleEnv), [ModuleWarning])
-runModuleT env m = 
+runModuleT env m =
     runWriterT
   $ runExceptionT
   $ runStateT env
@@ -444,5 +441,3 @@ getSolverConfig :: ModuleM T.SolverConfig
 getSolverConfig  = ModuleT $ do
   me <- get
   return (meSolverConfig me)
-
-
