@@ -6,15 +6,15 @@
 -- Stability   :  provisional
 -- Portability :  portable
 
-{-# LANGUAGE DeriveGeneric #-}
-{-# LANGUAGE FlexibleInstances #-}
-{-# LANGUAGE FlexibleContexts #-}
-{-# LANGUAGE RecordWildCards #-}
-{-# LANGUAGE DeriveTraversable #-}
+{-# LANGUAGE DeriveAnyClass #-}
 {-# LANGUAGE DeriveFoldable #-}
 {-# LANGUAGE DeriveFunctor #-}
+{-# LANGUAGE DeriveGeneric #-}
+{-# LANGUAGE DeriveTraversable #-}
+{-# LANGUAGE FlexibleContexts #-}
+{-# LANGUAGE FlexibleInstances #-}
 {-# LANGUAGE PatternGuards #-}
-
+{-# LANGUAGE RecordWildCards #-}
 module Cryptol.ModuleSystem.NamingEnv where
 
 import Cryptol.ModuleSystem.Interface
@@ -26,11 +26,12 @@ import Cryptol.Utils.Panic (panic)
 
 import Data.List (nub)
 import Data.Maybe (catMaybes,fromMaybe)
-import qualified Data.Map as Map
+import qualified Data.Map.Strict as Map
 import qualified Data.Set as Set
+import MonadLib (runId,Id)
 
 import GHC.Generics (Generic)
-import Control.DeepSeq.Generics
+import Control.DeepSeq
 
 import Prelude ()
 import Prelude.Compat
@@ -40,15 +41,13 @@ import Prelude.Compat
 
 -- XXX The fixity environment should be removed, and Name should include fixity
 -- information.
-data NamingEnv = NamingEnv { neExprs :: Map.Map PName [Name]
+data NamingEnv = NamingEnv { neExprs :: !(Map.Map PName [Name])
                              -- ^ Expr renaming environment
-                           , neTypes :: Map.Map PName [Name]
+                           , neTypes :: !(Map.Map PName [Name])
                              -- ^ Type renaming environment
-                           , neFixity:: Map.Map Name Fixity
+                           , neFixity:: !(Map.Map Name Fixity)
                              -- ^ Expression-level fixity environment
-                           } deriving (Show, Generic)
-
-instance NFData NamingEnv where rnf = genericRnf
+                           } deriving (Show, Generic, NFData)
 
 instance Monoid NamingEnv where
   mempty        =
@@ -68,6 +67,11 @@ instance Monoid NamingEnv where
               , neTypes  = Map.unionsWith merge (map neTypes  envs)
               , neFixity = Map.unions           (map neFixity envs) }
 
+  {-# INLINE mempty #-}
+  {-# INLINE mappend #-}
+  {-# INLINE mconcat #-}
+
+
 -- | Merge two name maps, collapsing cases where the entries are the same, and
 -- producing conflicts otherwise.
 merge :: [Name] -> [Name] -> [Name]
@@ -170,25 +174,44 @@ data InModule a = InModule !ModName a
 
 -- | Generate a 'NamingEnv' using an explicit supply.
 namingEnv' :: BindsNames a => a -> Supply -> (NamingEnv,Supply)
-namingEnv' a supply = runSupplyM supply (namingEnv a)
+namingEnv' a supply = runId (runSupplyT supply (runBuild (namingEnv a)))
+
+
+newtype BuildNamingEnv = BuildNamingEnv { runBuild :: SupplyT Id NamingEnv }
+
+instance Monoid BuildNamingEnv where
+  mempty = BuildNamingEnv (pure mempty)
+
+  mappend (BuildNamingEnv a) (BuildNamingEnv b) = BuildNamingEnv $
+    do x <- a
+       y <- b
+       return (mappend x y)
+
+  mconcat bs = BuildNamingEnv $
+    do ns <- sequence (map runBuild bs)
+       return (mconcat ns)
 
 -- | Things that define exported names.
 class BindsNames a where
-  namingEnv :: a -> SupplyM NamingEnv
+  namingEnv :: a -> BuildNamingEnv
 
 instance BindsNames NamingEnv where
-  namingEnv = return
+  namingEnv env = BuildNamingEnv (return env)
+  {-# INLINE namingEnv #-}
 
 instance BindsNames a => BindsNames (Maybe a) where
   namingEnv = foldMap namingEnv
+  {-# INLINE namingEnv #-}
 
 instance BindsNames a => BindsNames [a] where
   namingEnv = foldMap namingEnv
+  {-# INLINE namingEnv #-}
 
 -- | Generate a type renaming environment from the parameters that are bound by
 -- this schema.
 instance BindsNames (Schema PName) where
   namingEnv (Forall ps _ _ _) = foldMap namingEnv ps
+  {-# INLINE namingEnv #-}
 
 
 -- | Interpret an import in the context of an interface, to produce a name
@@ -240,14 +263,15 @@ data ImportIface = ImportIface Import Iface
 -- | Produce a naming environment from an interface file, that contains a
 -- mapping only from unqualified names to qualified ones.
 instance BindsNames ImportIface where
-  namingEnv (ImportIface imp Iface { .. }) =
+  namingEnv (ImportIface imp Iface { .. }) = BuildNamingEnv $
     return (interpImport imp ifPublic)
+  {-# INLINE namingEnv #-}
 
--- | Introduce the name 
+-- | Introduce the name
 instance BindsNames (InModule (Bind PName)) where
-  namingEnv (InModule ns b) =
+  namingEnv (InModule ns b) = BuildNamingEnv $
     do let Located { .. } = bName b
-       n <- liftSupply (mkDeclared ns (getIdent thing) srcRange)
+       n <- liftSupply (mkDeclared ns (getIdent thing) (bFixity b) srcRange)
 
        let fixity = case bFixity b of
              Just f  -> mempty { neFixity = Map.singleton n f }
@@ -257,7 +281,7 @@ instance BindsNames (InModule (Bind PName)) where
 
 -- | Generate the naming environment for a type parameter.
 instance BindsNames (TParam PName) where
-  namingEnv TParam { .. } =
+  namingEnv TParam { .. } = BuildNamingEnv $
     do let range = fromMaybe emptyRange tpRange
        n <- liftSupply (mkParameter (getIdent tpName) range)
        return (singletonT tpName n)
@@ -274,20 +298,20 @@ instance BindsNames (InModule (TopDecl PName)) where
     case td of
       Decl d      -> namingEnv (InModule ns (tlValue d))
       TDNewtype d -> namingEnv (InModule ns (tlValue d))
-      Include _   -> return mempty
+      Include _   -> mempty
 
 instance BindsNames (InModule (Newtype PName)) where
-  namingEnv (InModule ns Newtype { .. }) =
+  namingEnv (InModule ns Newtype { .. }) = BuildNamingEnv $
     do let Located { .. } = nName
-       tyName <- liftSupply (mkDeclared ns (getIdent thing) srcRange)
-       eName  <- liftSupply (mkDeclared ns (getIdent thing) srcRange)
+       tyName <- liftSupply (mkDeclared ns (getIdent thing) Nothing srcRange)
+       eName  <- liftSupply (mkDeclared ns (getIdent thing) Nothing srcRange)
        return (singletonT thing tyName `mappend` singletonE thing eName)
 
 -- | The naming environment for a single declaration.
 instance BindsNames (InModule (Decl PName)) where
   namingEnv (InModule pfx d) = case d of
-    DBind b ->
-      do n <- mkName (bName b)
+    DBind b -> BuildNamingEnv $
+      do n <- mkName (bName b) (bFixity b)
          return (singletonE (thing (bName b)) n `mappend` fixity n b)
 
     DSignature ns _sig    -> foldMap qualBind ns
@@ -299,15 +323,15 @@ instance BindsNames (InModule (Decl PName)) where
 
     where
 
-    mkName ln =
-      liftSupply (mkDeclared pfx (getIdent (thing ln)) (srcRange ln))
+    mkName ln fx =
+      liftSupply (mkDeclared pfx (getIdent (thing ln)) fx (srcRange ln))
 
-    qualBind ln =
-      do n <- mkName ln
+    qualBind ln = BuildNamingEnv $
+      do n <- mkName ln Nothing
          return (singletonE (thing ln) n)
 
-    qualType ln =
-      do n <- mkName ln
+    qualType ln = BuildNamingEnv $
+      do n <- mkName ln Nothing
          return (singletonT (thing ln) n)
 
     fixity n b =
