@@ -7,35 +7,30 @@
 -- Portability :  portable
 
 {-# LANGUAGE Safe #-}
+
+{-# LANGUAGE DeriveAnyClass #-}
 {-# LANGUAGE DeriveGeneric #-}
 {-# LANGUAGE RecordWildCards #-}
-
 module Cryptol.Parser.Position where
 
 import           Data.Text.Lazy (Text)
 import qualified Data.Text.Lazy as T
 
 import GHC.Generics (Generic)
-import Control.DeepSeq.Generics
+import Control.DeepSeq
 
 import Cryptol.Utils.PP
 
 data Located a  = Located { srcRange :: !Range, thing :: a }
-                  deriving (Eq,Show,Generic)
-
-instance NFData a => NFData (Located a) where rnf = genericRnf
+                  deriving (Eq, Show, Generic, NFData)
 
 data Position   = Position { line :: !Int, col :: !Int }
-                  deriving (Eq,Ord,Show,Generic)
-
-instance NFData Position where rnf = genericRnf
+                  deriving (Eq, Ord, Show, Generic, NFData)
 
 data Range      = Range { from   :: !Position
                         , to     :: !Position
                         , source :: FilePath }
-                  deriving (Eq,Show,Generic)
-
-instance NFData Range where rnf = genericRnf
+                  deriving (Eq, Show, Generic, NFData)
 
 -- | An empty range.
 --
@@ -80,10 +75,9 @@ instance PP a => PP (Located a) where
   ppPrec _ l = parens (text "at" <+> pp (srcRange l) <> comma <+> pp (thing l))
 
 instance PPName a => PPName (Located a) where
-  ppPrefixName Located { .. } = ppPrefixName thing
-  ppInfixName  Located { .. } = ppInfixName  thing
-
-
+  ppNameFixity  Located { .. } = ppNameFixity thing
+  ppPrefixName  Located { .. } = ppPrefixName thing
+  ppInfixName   Located { .. } = ppInfixName  thing
 
 --------------------------------------------------------------------------------
 
