@@ -1,22 +1,21 @@
 -- |
 -- Module      :  $Header$
--- Copyright   :  (c) 2015 Galois, Inc.
+-- Copyright   :  (c) 2015-2016 Galois, Inc.
 -- License     :  BSD3
 -- Maintainer  :  cryptol@galois.com
 -- Stability   :  provisional
 -- Portability :  portable
 --
--- Include the prelude when building with -fself-contained
+-- Compile the prelude into the executable as a last resort
 
 {-# LANGUAGE CPP #-}
 {-# LANGUAGE QuasiQuotes #-}
+{-# LANGUAGE OverloadedStrings #-}
 
 module Cryptol.Prelude (writePreludeContents) where
 
 import Cryptol.ModuleSystem.Monad
 
-#ifdef SELF_CONTAINED
-
 import System.Directory (getTemporaryDirectory)
 import System.IO (hClose, hPutStr, openTempFile)
 import Text.Heredoc (there)
@@ -33,13 +32,3 @@ writePreludeContents = io $ do
   hPutStr h preludeContents
   hClose h
   return path
-
-#else
-
-import Cryptol.Parser.AST as P
-
--- | If we're not self-contained, the Prelude is just missing
-writePreludeContents :: ModuleM FilePath
-writePreludeContents = moduleNotFound (P.ModName ["Cryptol"]) =<< getSearchPath
-
-#endif
