@@ -245,9 +245,10 @@ destETAbs = go []
 freshName :: Name -> [Type] -> SpecM Name
 freshName n _ =
   case nameInfo n of
-    Declared ns -> liftSupply (mkDeclared ns ident loc)
+    Declared ns -> liftSupply (mkDeclared ns ident fx loc)
     Parameter   -> liftSupply (mkParameter ident loc)
   where
+  fx    = nameFixity n
   ident = nameIdent n
   loc   = nameLoc n
 
